import createVanilla from 'zustand/vanilla';
import create from 'zustand';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var undoStore = /*#__PURE__*/createVanilla(function (_, get) {
  return {
    prevStates: [],
    undo: function undo() {
      var prevStates = get().prevStates;

      if (prevStates.length > 0) {
        var prevState = prevStates.pop();
        get().futureStates.push(get().getStore());
        get().setStore(prevState);
      }
    },
    setStore: function setStore() {},
    getStore: function getStore() {},
    futureStates: [],
    redo: function redo() {
      var futureStates = get().futureStates;

      if (futureStates.length > 0) {
        var futureState = futureStates.pop();
        get().prevStates.push(get().getStore());
        get().setStore(futureState);
      }
    }
  };
});
var getState = undoStore.getState,
    setState = undoStore.setState;
var useUndo = /*#__PURE__*/create(undoStore); // custom middleware to get previous state

var undo = function undo(config) {
  return function (set, get, api) {
    return config(function (args) {
      setState({
        prevStates: [].concat(getState().prevStates, [_extends({}, get())]),
        setStore: set,
        futureStates: [],
        getStore: get
      });
      set(args);
    }, get, api);
  };
};

export { undo, useUndo };
//# sourceMappingURL=zundo.esm.js.map
